import androidx.compose.animation.ExperimentalSharedTransitionApi
import androidx.compose.animation.SharedTransitionLayout
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.WindowInsets
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.systemBars
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.windowInsetsPadding
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.rememberLazyGridState
import androidx.compose.material.Button
import androidx.compose.material.Scaffold
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navArgument
import details.RecipeDetails
import recipeslist.RecipesListScreen
import sensor.SensorManager
import ui.theme.MainTheme
import view.FilterCard
import view.HomeScreen
import view.InputFieldState
import view.getFilteredRecipeList
import view.getRecipeList


enum class Theme {
    Auto,
    Light,
    Dark
}

enum class RecipeAppScreen {
    List, Details,
}

@OptIn(ExperimentalSharedTransitionApi::class)
@Composable
fun App(sensorManager: SensorManager?, isLarge: Boolean = false) {

    val navController = rememberNavController()
    var theme by remember { mutableStateOf(Theme.Auto) }

    val onThemeToggle = {
        theme = when (theme) {
            Theme.Auto -> Theme.Light
            Theme.Light -> Theme.Dark
            Theme.Dark -> Theme.Auto
        }
    }
    val isDarkTheme: Boolean? = when (theme) {
        Theme.Dark -> true
        Theme.Light -> false
        Theme.Auto -> null
    }

    MainTheme(isDarkTheme ?: isSystemInDarkTheme()) {

        var show by remember { mutableStateOf(false) }
        var searchBar by remember { mutableStateOf(false) }
        var search by remember { mutableStateOf("") }
        val tags = remember { mutableStateListOf<String>() }
        val recipeTags by remember { mutableStateOf(mutableListOf<String>()) }
        var book by remember { mutableStateOf("") }

        for (recipe in getRecipeList()) {
            for (tag in recipe.tags) {
                if (!recipeTags.contains(tag))
                    recipeTags.add(tag)
            }
        }

        if (show) {
            FilterCard(recipeTags) {
                tags.removeAll(tags)
                tags.addAll(it)
                show = false
            }
        }

        val filteredItems = getFilteredRecipeList(tags, search, book)
        var currentRecipe = getRecipeList().first()

        if (filteredItems.isNotEmpty())
            currentRecipe = filteredItems.first()

        Scaffold {
            Column {
                if (searchBar) {
                    Row {
                        Button(
                            onClick = {
                                show = true
                            },
                        ) {
                            Text(text = "Filter")
                        }
                        InputFieldState(
                            value = search,
                            label = "Search",
                            modifier = Modifier
                                .width(300.dp)
                        ) {
                            search = it
                        }
                    }
                }

                SharedTransitionLayout {
                    NavHost(
                        navController = navController,
                        startDestination = "Home",
                        modifier = Modifier.fillMaxSize()
                    ) {
                        composable(route = "Home")
                        {
                            HomeScreen(
                                isLarge = isLarge,
                                onClick = {
                                    searchBar = false
                                    navController.navigate("BookShelf")
                                }
                            )
                        }
                        composable(route = "BookShelf")
                        {
                            BookShelf(
                                isLarge = isLarge,
                                tags = recipeTags,
                                onClick = { lockedTag ->
                                    book = lockedTag
                                    searchBar = true
                                    navController.navigate(RecipeAppScreen.List.name)

                                }
                            )
                        }
                        composable(
                            route = RecipeAppScreen.List.name
                            ) {
                            RecipesListScreen(
                                isLarge = isLarge,
                                items = filteredItems,
                                onClick = { recipe ->
                                    searchBar = false
                                    currentRecipe = recipe
                                    navController.navigate(RecipeAppScreen.Details.name)
                                })
                        }
                        composable(route = RecipeAppScreen.Details.name) {
                            RecipeDetails(
                                isLarge = isLarge,
                                sensorManager = sensorManager,
                                recipe = currentRecipe,
                                goBack = {
                                    searchBar = true
                                    navController.popBackStack()
                                })
                        }
                    }
                }
            }
        }
    }
}

@Composable
fun BookShelf(
    onClick: (String) -> Unit,
    isLarge: Boolean,
    tags: List<String>
) {

    Box(
        modifier = Modifier
            .fillMaxSize()
    ) {
        Column(
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 10.dp)
                .align(Alignment.Center),
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.Center,
        ) {

            Text(text = "What Do You Want To Cook Today?", textAlign = TextAlign.Center)

            Button(onClick = { onClick("") }) {
                Text("ANYTHING!")
            }

            val listState = rememberLazyGridState()
            LazyVerticalGrid(
                columns = GridCells.Fixed(if(isLarge) 4 else 2),
                state = listState
            )
            {
                if(isLarge.not()) {
                    item {
                        Spacer(modifier = Modifier.windowInsetsPadding(WindowInsets.systemBars))
                    }
                    items(tags.size) {item ->
                        val tag = tags[item]
                        Button(onClick = {onClick(tag) }) {
                            Text(tag)
                        }
                    }
                }

            }

        }
    }
}
